package es.verdnatura.presentation.view.feature.ubicador.adapter

import android.annotation.SuppressLint
import android.provider.Settings.Global.getString
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import es.verdnatura.R
import es.verdnatura.databinding.ItemUbicationRowBinding
import es.verdnatura.presentation.common.OnMoreClickListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.common.OnVisibleClickListener
import es.verdnatura.presentation.common.loadUrl
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.ubicador.model.ItemUbicadorVO

class UbicadorAdapter (
    private val items: List<ItemUbicadorVO>,
    private val onPasillerosItemClickListener: OnPasillerosItemClickListener,
    private val onVisibleClickListener: OnVisibleClickListener,
    private val onMoreClickListener: OnMoreClickListener
): RecyclerView.Adapter<UbicadorAdapter.ItemHolder> () {


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ItemHolder {
        return ItemHolder(
            ItemUbicationRowBinding.inflate(LayoutInflater.from(parent.context),parent,false)
        )
    }

    override fun getItemCount() =items.size

    override fun onBindViewHolder(holder: ItemHolder, position: Int) {
        holder.bind(items[position])
    }

    inner class ItemHolder(
        val binding: ItemUbicationRowBinding
    ) : RecyclerView.ViewHolder(binding.root){
        private val res = binding.root.context.resources
        @SuppressLint("SetTextI18n")
        fun bind(item: ItemUbicadorVO) {
            binding.apply {
                this.item = item
                multiText.text = "${item.stickers}x${item.packing}"
                imgItem.loadUrl("https://verdnatura.es/vn-image-data/catalog/200x200/"+item.item)
                imgItem.setOnClickListener {
                    onPasillerosItemClickListener.onPasillerosItemClickListener(PasillerosItemVO(title = res.getString(
                                            R.string.itemSearch)),item.item)
                }

                layerVisible.setOnClickListener {
                    onVisibleClickListener.onVisibleClickListener(item)
                }

                if (item.visible == "0"){
                    visibleText.setTextColor(ContextCompat.getColor(visibleText.context,R.color.verdnatura_brown_grey))

                }else{
                    visibleText.setTextColor(ContextCompat.getColor(visibleText.context,R.color.verdnatura_pumpkin_orange))
                }

                moreImg.setOnClickListener {
                    onMoreClickListener.onMoreClickListener(item)
                }
                namelayout.setOnClickListener {
                    onPasillerosItemClickListener.onPasillerosItemClickListener(PasillerosItemVO(title = res.getString(R.string.itemSearch)),item.item)
                }

            }
        }
    }
}